/* This is the interface to the python interpreter
   Originally from the now defunct gnocci project.
   C. Steenberg 15 May 2000
*/

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>
#include <string.h>
#include <Python.h>
#include <grammar.h>
#include <node.h>
#include <parsetok.h>
#include <errcode.h>
#include <compile.h>
#include <eval.h>
#include <marshal.h>
#ifdef WITH_NUMERIC_PYTHON
#include <arrayobject.h>
#endif
#include "../main.h"
#include "python_int.h"
#include "import_data.h"


PyObject *eval_dict, *main_o, *dict_o;
extern grammar _PyParser_Grammar; /* From Python/graminit.c */
extern pid_t child_pid;
SGworksheet *sg_current_worksheet;

/* Function forward declarations */
static PyObject *get_cell(PyObject *self, PyObject *args, PyObject *keywds);
static PyObject *get_col(PyObject *self, PyObject *args, PyObject *keywds);
static PyObject *get_row(PyObject *self, PyObject *args, PyObject *keywds);

PyObject *set_sheet(gint row, gint col, PyObject *object, SGworksheet *worksheet,
                     GtkOrientation orient);

static PyObject *set_cell(PyObject *self, PyObject *args, PyObject *keywds);
static PyObject *set_col(PyObject *self, PyObject *args, PyObject *keywds);
static PyObject *set_row(PyObject *self, PyObject *args, PyObject *keywds);


SGworksheet *sg_sheet_get_by_name(GList *worksheets,gchar *name)
{ SGworksheet *sheet;
  GList *list;

  list=worksheets;
  if (!strcmp(name,"current") && sg_current_worksheet!=NULL)
    { sheet=sg_current_worksheet;
      return sheet;
    }
  else

  while (list)
  { sheet=(SGworksheet *)(list->data);
    if (!strcmp(sheet->name,name)) return (sheet);

    list=list->next;
  }
/* Oh well, return first sheet in list */
  if (worksheets) sheet=(SGworksheet *)(worksheets->data);
  else {
  PyErr_SetString(PyExc_ValueError,"Could not determine sheet reference");
  }
  return sheet;
}

static PyMethodDef SGMethods[] =
    {
        {"cell",  (PyCFunction)get_cell, METH_VARARGS|METH_KEYWORDS},
        {"col",  (PyCFunction)get_col, METH_VARARGS|METH_KEYWORDS},
        {"row",  (PyCFunction)get_row, METH_VARARGS|METH_KEYWORDS},
        {"setcell",  (PyCFunction)set_cell, METH_VARARGS|METH_KEYWORDS},
        {"setcol",  (PyCFunction)set_col, METH_VARARGS|METH_KEYWORDS},
        {"setrow",  (PyCFunction)set_row, METH_VARARGS|METH_KEYWORDS},
        {"read_block", (PyCFunction)python_read_table_block, METH_VARARGS|METH_KEYWORDS},
        {"read_lines", (PyCFunction)python_read_table_lines, METH_VARARGS|METH_KEYWORDS},
        {NULL, NULL}        /* Sentinel */
    };

void init_sg_python()
{ (void) Py_InitModule("sg", SGMethods);
}

gint get_worksheet_col(SGworksheet *worksheet,PyObject *col_obj)
{ gint col,num;
  gchar *col_name;

  if (PyString_Check(col_obj))
      { num=gtk_sheet_get_columns_count(GTK_SHEET(worksheet->sheet));
	col_name=PyString_AsString(col_obj);
	for (col=0;col<num;col++)
	    if (GTK_SHEET(worksheet->sheet)->column[col].name)
	     { if (!strcmp(col_name,GTK_SHEET(worksheet->sheet)->column[col].name))
	       break;
	     }
	    else
	      { if (col==atoi(col_name));
		return col;
	      }
      }
  else if (PyInt_Check(col_obj))
      col=(int)PyInt_AsLong(col_obj);

  /* Now check if row and col values are sane */
  if (col<0 || col>gtk_sheet_get_columns_count(GTK_SHEET(worksheet->sheet)))
  { PyErr_SetString(PyExc_ValueError,"Column number out of range");
    return -1;
  }
  return(col);
}

gint get_worksheet_row(SGworksheet *worksheet,PyObject *row_obj)
{ gint row,num;
  gchar *row_name;

  if (PyString_Check(row_obj))
      { num=gtk_sheet_get_rows_count(GTK_SHEET(worksheet->sheet));
	row_name=PyString_AsString(row_obj);
	  { for (row=0;row<num;row++)
             if (GTK_SHEET(worksheet->sheet)->row[row].name)
	      { if (!strcmp(row_name,GTK_SHEET(worksheet->sheet)->row[row].name))
	         break;
	      }
	     else
	      { if (row==atoi(row_name));
		return(row);
	      }
	  }
      }
  else if (PyInt_Check(row_obj))
      row=(int)PyInt_AsLong(row_obj);

  /* Now check if row and col values are sane */
  if (row<0 || row>gtk_sheet_get_rows_count(GTK_SHEET(worksheet->sheet)))
  { PyErr_SetString(PyExc_ValueError,"Row number out of range");
    return -1;
  }
  return(row);
}

/* Cell/Col/Row query functions */

static PyObject *get_cell(PyObject *self, PyObject *args, PyObject *keywds)
{ gchar *sheet="current",*content;
  static gchar *kwlist[]={"row","col","sheet",NULL};
  gint row=0,col=0,num,arow=-1,acol=-1;
  SGworksheet *worksheet=NULL;
  gdouble value;
  PyObject *object,*row_obj,*col_obj;
  gboolean restore_active=FALSE;

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "OO|s", kwlist,
                                         &row_obj, &col_obj, &sheet))
      return NULL;
/* Get sheet pointer */
  worksheet=sg_sheet_get_by_name(worksheets,sheet);
  if (!worksheet) return NULL;

  /* Now check if row and col values are sane */
  row=get_worksheet_row(worksheet,row_obj);
  if (row<0) return NULL;
  col=get_worksheet_col(worksheet,col_obj);
  if (col<0) return NULL;


  content=gtk_sheet_cell_get_text(GTK_SHEET(worksheet->sheet),row,col);
  if (content)
      num=sscanf(content,"%lf",&value);
  else num=0;

  if (num)
    object=Py_BuildValue("f", value);
  else
    object=Py_BuildValue("s",content);

  Py_INCREF(object);
  return object;
}


static PyObject *get_col(PyObject *self, PyObject *args, PyObject *keywds)
{ gchar *sheet="current",*content;
  static gchar *kwlist[]={"col","sheet",NULL},*col_name;
  gint row=0,num,col=0,nd=1,dims[1],i,j=0,arow=-1,acol=-1;
  SGworksheet *worksheet=NULL;
  gdouble value,*data,*rdata;
  PyObject *object,*col_obj;
  gboolean restore_active=FALSE;

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|s", kwlist,
                                         &col_obj, &sheet))
      return NULL;
/* Get sheet pointer */
  worksheet=sg_sheet_get_by_name(worksheets,sheet);
  if (!worksheet) return NULL;

  col=get_worksheet_col(worksheet,col_obj);
  if (col<0) return NULL;


/* See how many elements we need */
  data=g_new(gdouble,GTK_SHEET(worksheet->sheet)->maxallocrow+1);
  for (i=0;i<=GTK_SHEET(worksheet->sheet)->maxallocrow;i++)
    { content=gtk_sheet_cell_get_text(GTK_SHEET(worksheet->sheet),i,col);
      if (content)
       { num=sscanf(content,"%lf",&value);
         if (num) data[j++]=value;
       }
    }
  if (i==j) rdata=data;
  else
  {
      rdata=g_new(gdouble,j);
      for (i=0;i<j;i++)
          rdata[i]=data[i];
      g_free(data);
   }


  if (j>0)
  {
      dims[0]=j;
      object=PyArray_FromDimsAndData(1,dims,PyArray_DOUBLE,(gchar *)rdata);
  }
  else object=Py_None;

  Py_INCREF(object);
  return object;
}

static PyObject *get_row(PyObject *self, PyObject *args, PyObject *keywds)
{ gchar *sheet="current",*content;
  static gchar *kwlist[]={"row","sheet",NULL};
  gint row=0,arow=-1,acol=-1,num,nd=1,dims[1],i,j=0;
  SGworksheet *worksheet=NULL;
  gdouble value,*data,*rdata;
  PyObject *object, *row_obj;
  gboolean restore_active=FALSE;

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|s", kwlist,
                                         &row_obj, &sheet))
      return NULL;
  /* Get sheet pointer */
  worksheet=sg_sheet_get_by_name(worksheets,sheet);
  if (!worksheet) return NULL;

  /* Now check if row and col values are sane */
  row=get_worksheet_row(worksheet,row_obj);
  if (row<0) return NULL;

/* See how many elements we need */
  data=g_new(gdouble,GTK_SHEET(worksheet->sheet)->maxalloccol+1);
  for (i=0;i<=GTK_SHEET(worksheet->sheet)->maxalloccol;i++)
    { content=gtk_sheet_cell_get_text(GTK_SHEET(worksheet->sheet),row,i);
      if (content)
       { num=sscanf(content,"%lf",&value);
         if (num) data[j++]=value;
       }
    }


  if (i==j) rdata=data;
  else
   { rdata=g_new(gdouble,j);
     for (i=0;i<j;i++)
         rdata[i]=data[i];
     g_free(data);
   }

   if (j>0)
   {
       dims[0]=j;
       object=PyArray_FromDimsAndData(1,dims,PyArray_DOUBLE,(gchar *)rdata);
   }
   else object=Py_None;


  Py_INCREF(object);
  return object;
}

PyObject *set_sheet(gint row, gint col, PyObject *object, SGworksheet *worksheet,
                     GtkOrientation orient)
{ gint retval,acol,arow;
  gboolean restore_active=FALSE;

  /* Now check if row and col values are sane */

#ifdef WITH_NUMERIC_PYTHON
      if (PyArray_Check(object))
       { gtk_sheet_freeze(GTK_SHEET(worksheet->sheet));
         retval=python_array(worksheet,row,col,(PyArrayObject *)object,orient);
         gtk_sheet_thaw(GTK_SHEET(worksheet->sheet));
       }
      else
#endif
      if (PySequence_Check(object))
       { gtk_sheet_freeze(GTK_SHEET(worksheet->sheet));
         retval=python_sequence(worksheet,row,col,object,orient);
         gtk_sheet_thaw(GTK_SHEET(worksheet->sheet));
       }

      else
        retval=python_singleton(worksheet,row,col,object,orient,FALSE);


      if (!retval)
      { if (PyErr_Occurred())
          { if (child_pid>0)
               { PyErr_Print();
                 if (Py_FlushLine()) PyErr_Clear();
               }
            PyErr_Clear();
          }
         return NULL;
      }
      else
      {   Py_INCREF(Py_None);
          return Py_None;
      }
}


static PyObject *set_cell(PyObject *self, PyObject *args, PyObject *keywds)
{ gchar *sheet="current",*content;
  static gchar *kwlist[]={"row","col","object","sheet",NULL};
  gint row=0,col=0,num,arow=-1,acol=-1,retval;
  SGworksheet *worksheet=NULL;
  gdouble value;
  PyObject *object,*row_obj,*col_obj;
  gboolean restore_active=FALSE;

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "OOO|s", kwlist,
                                         &row_obj, &col_obj, &object,&sheet))
    return NULL;
  worksheet=sg_sheet_get_by_name(worksheets,sheet);
  if (!worksheet) return NULL;

  row=get_worksheet_row(worksheet,row_obj);
  if (row<0) return NULL;
  col=get_worksheet_col(worksheet,col_obj);
  if (col<0) return NULL;

  return set_sheet(row,col,object,worksheet,GTK_ORIENTATION_VERTICAL);
}

static PyObject *set_col(PyObject *self, PyObject *args, PyObject *keywds)
{ gchar *sheet="current",*content;
  static gchar *kwlist[]={"col","object","sheet",NULL};
  gint row=0,col=0,num,arow=-1,acol=-1,retval;
  SGworksheet *worksheet=NULL;
  gdouble value;
  PyObject *object,*col_obj;
  gboolean restore_active=FALSE;

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "OO|s", kwlist,
                                         &col_obj, &object,&sheet))
      return NULL;
  worksheet=sg_sheet_get_by_name(worksheets,sheet);
  if (!worksheet) return NULL;

  col=get_worksheet_col(worksheet,col_obj);
  if (col<0) return NULL;

  return set_sheet(0,col,object,worksheet,GTK_ORIENTATION_VERTICAL);
}

static PyObject *set_row(PyObject *self, PyObject *args, PyObject *keywds)
{ gchar *sheet="current",*content;
  static gchar *kwlist[]={"row","object","sheet",NULL};
  gint row=0,col=0,num,arow=-1,acol=-1,retval;
  SGworksheet *worksheet=NULL;
  gdouble value;
  PyObject *object,*row_obj;
  gboolean restore_active=FALSE;

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "OO|s", kwlist,
                                         &row_obj, &object,&sheet))
      return NULL;

  worksheet=sg_sheet_get_by_name(worksheets,sheet);
  if (!worksheet) return NULL;

  row=get_worksheet_row(worksheet,row_obj);
  if (row<0) return NULL;

  return set_sheet(row,0,object,worksheet,GTK_ORIENTATION_HORIZONTAL);
}

int python_init(gchar *progname)
{   Py_SetProgramName(progname);
    Py_Initialize();
    PyRun_SimpleString ("import sys");
    PyRun_SimpleString ("import os");
    eval_dict = Py_BuildValue("{s{}}", "__builtins__");
    main_o=PyImport_AddModule("__main__");
    dict_o=PyModule_GetDict(main_o);
/* Numerical python */
#ifdef WITH_NUMERIC_PYTHON
    PyRun_SimpleString ("from Numeric import *");
#endif
/* sg module init */
    init_sg_python();
    PyRun_SimpleString ("from sg import *");
    return(1);
}

int python_singleton(SGworksheet *worksheet,gint row, gint col,
                     PyObject *object,gboolean link)
{ static PyObject *s_obj;
  GtkSheet *sheet;
  gchar *string;

  sheet = GTK_SHEET(worksheet->sheet);

  s_obj=PyObject_Str(object);

  if (s_obj)
   { Py_INCREF(s_obj);
     if (object==Py_None)
         string=strdup("");
     else
         string=PyString_AsString (s_obj);
   if (link)
       sg_worksheet_cell_set(worksheet,row,col,string,TRUE,FALSE);
   else
       gtk_sheet_set_cell_text(GTK_SHEET(worksheet->sheet),row,col,string);
   Py_DECREF(s_obj);
   }
  return 1;
}

int python_sequence(SGworksheet *worksheet,gint row, gint col, PyObject *object,
                    GtkOrientation orient,gboolean link)
{ gint len,i;
  PyObject *seq_obj;
  GtkSheet *sheet;

  sheet = GTK_SHEET(worksheet->sheet);

  len=PySequence_Length(object);

  /* Check to see if we need to resize the sheet */
  seq_obj=PySequence_GetItem(object,0);
  if (orient==GTK_ORIENTATION_VERTICAL)
  {   if (PySequence_Check(seq_obj) && !PyString_Check(seq_obj) &&
          gtk_sheet_get_columns_count(sheet) < len+col+1)
           sg_worksheet_add_columns(worksheet,len-
                gtk_sheet_get_columns_count(sheet)+col-1);
      else if (!PySequence_Check(seq_obj) && gtk_sheet_get_rows_count(sheet) < row+len)
            sg_worksheet_add_rows(worksheet,len-gtk_sheet_get_rows_count(sheet)+row-1);
  }
  else
  {   if (PySequence_Check(seq_obj) && !PyString_Check(seq_obj) &&
          gtk_sheet_get_rows_count(sheet) <
          len+row+1)
           sg_worksheet_add_rows(worksheet,len-
                gtk_sheet_get_rows_count(sheet)+row-1);
      else if (!PySequence_Check(seq_obj) && gtk_sheet_get_columns_count(sheet) < col+len)
            sg_worksheet_add_columns(worksheet,len-gtk_sheet_get_columns_count(sheet)+col-1);
  }

  for (i=0;i<len;i++)
    { seq_obj=PySequence_GetItem(object,i);
      Py_INCREF(seq_obj);
      if (PySequence_Check(seq_obj) && !PyString_Check(seq_obj))
      {  if (orient==GTK_ORIENTATION_VERTICAL)
             python_sequence(worksheet,row,col+i,seq_obj,orient,link);
         else
             python_sequence(worksheet,row+i,col,seq_obj,orient,link);
      }
      else
      { if (orient==GTK_ORIENTATION_VERTICAL)
         python_singleton(worksheet,row+i,col,seq_obj,link);
      else
          python_singleton(worksheet,row,col+i,seq_obj,link);
      }
      Py_XDECREF(seq_obj);
    }
  return 1;
}

#ifdef WITH_NUMERIC_PYTHON
int python_array_print(SGworksheet *worksheet,gint row, gint col,
                       PyArrayObject *object,gint dim, gchar *data,
                       GtkOrientation orient,gboolean link)
{ gchar *array,singleton[80],CHAR[]="%%%dc",UBYTE[]="%%%dhi",
        SHORT[]="%%%dhd",
        INT[]="%%%dd",LONG[]="%%%dld",FLOAT[]="%%g",CFLOAT[]="%%Lg",
        format[20],*fpt,*apt;
  gint stride,i,j,vert,rowskip;

  
  array=data;
  if (dim>1)
   { rowskip=1;
     for (i=dim;i>0;i--)
       rowskip*=object->dimensions[i];
     stride=object->strides[dim];
     for (i=0;i<object->dimensions[i];i++)
       { python_array_print(worksheet,row+i*rowskip,col,object,dim-1,array,orient,link);
         array+=stride;
       }
   }
  else
    { if (dim>0)
       { vert=object->dimensions[1];
         stride=object->strides[1];
       }
      else
       { vert=1;
         stride=0;
       }
  /* Now insert the cell values */

      array=data;
      switch (object->descr->type_num)
       { case PyArray_CHAR: fpt=CHAR;
          break;
         case PyArray_UBYTE: fpt=UBYTE;
          break;
         case PyArray_SBYTE:
         case PyArray_SHORT: fpt=SHORT;
          break;
         case PyArray_INT: fpt=INT;
          break;
         case PyArray_LONG: fpt=LONG;
          break;
         case PyArray_FLOAT:
         case PyArray_DOUBLE: fpt=FLOAT;
          break;
         case PyArray_CFLOAT:
         case PyArray_CDOUBLE: fpt=CFLOAT;
          break;
       }
       snprintf(format,20,fpt,worksheet->column[col].precision);
      for (j=0;j<vert;j++)
        { for (i=0;i<object->dimensions[0];i++)
            { apt=array+i*object->strides[0];
              switch (object->descr->type_num)
               { case PyArray_CHAR:
                 case PyArray_UBYTE:
                 case PyArray_SBYTE:
                  snprintf(singleton,80,format,*(char *)(array+i*object->strides[0]));
                  break;
                 case PyArray_SHORT:
                  snprintf(singleton,80,format,*(short *)(array+i*object->strides[0]));
                  break;
                 case PyArray_INT:
                  snprintf(singleton,80,format,*(int *)(array+i*object->strides[0]));
                  break;
                 case PyArray_LONG:
                  snprintf(singleton,80,format,*(long *)(array+i*object->strides[0]));
                  break;
                 case PyArray_FLOAT:
                  snprintf(singleton,80,format,*(float *)(array+i*object->strides[0]));
                  break;
                 case PyArray_DOUBLE:
                  snprintf(singleton,80,format,*(double *)(array+i*object->strides[0]));
                  break;
                 case PyArray_CFLOAT:
                 case PyArray_CDOUBLE:
                  snprintf(singleton,80,format,*(long double *)(array+i*object->strides[0]));
                  break;
                 case PyArray_OBJECT:
                  python_sequence(worksheet,row+j,col+i,
                   (PyObject *)(array+i*object->strides[0]),orient,link);
                  break;
               }
            if (link)
                {
                    if (orient==GTK_ORIENTATION_VERTICAL)
                        sg_worksheet_cell_set(worksheet,row+i,col+j,strdup(singleton),TRUE,FALSE);
                    else
                        sg_worksheet_cell_set(worksheet,row+j,col+i,strdup(singleton),TRUE,FALSE);
                }
            else
            {
                 if (orient==GTK_ORIENTATION_VERTICAL)
                     gtk_sheet_set_cell_text(GTK_SHEET(worksheet->sheet),row+i,col+j,strdup(singleton));
                 else
                     gtk_sheet_set_cell_text(GTK_SHEET(worksheet->sheet),row+j,col+i,strdup(singleton));
            }

          }
          array+=stride;
        }
    }
  return 1;
}

int python_array(SGworksheet *worksheet,gint row, gint col,
                 PyArrayObject *object, GtkOrientation orient,gboolean link)
{ gint len,i;
  GtkSheet *sheet;

  sheet = GTK_SHEET(worksheet->sheet);

  if (object->nd>2) return 0; /* No nore than 2 dimensions for now */

  /* Check to see if we need to resize the sheet */
  if (orient==GTK_ORIENTATION_VERTICAL)
  {
      if (object->nd>1 && 
          gtk_sheet_get_columns_count(sheet)< object->dimensions[1]+col+1)
         sg_worksheet_add_columns(worksheet,object->dimensions[1]-
                gtk_sheet_get_columns_count(sheet)+col);
      len=1;
      for (i=0;i<object->nd;i++)
        if (i!=1) len=len*object->dimensions[i];
    
      if (gtk_sheet_get_rows_count(sheet) < row+len)
          sg_worksheet_add_rows(worksheet,len-gtk_sheet_get_rows_count(sheet)+row);
  }
  else
  {
      if (object->nd>1 && 
          gtk_sheet_get_rows_count(sheet)< object->dimensions[1]+row+1)
         sg_worksheet_add_rows(worksheet,object->dimensions[1]-
                gtk_sheet_get_rows_count(sheet)+row);
      len=1;
      for (i=0;i<object->nd;i++)
        if (i!=1) len=len*object->dimensions[i];
    
      if (gtk_sheet_get_columns_count(sheet) < col+len)
          sg_worksheet_add_columns(worksheet,len-gtk_sheet_get_columns_count(sheet)+col);

  }

  python_array_print(worksheet,row,col,object,object->nd-1,object->data,
                     orient,link);

  return 1;
}

#endif

int python_insert_object(SGworksheet *worksheet,gint row, gint col,
                         PyObject *object,GtkOrientation orient,gboolean link)
{ gchar errmsg[]="Error!";
  gchar *msg;
  GtkSheet *sheet;
  SGworksheet *temp_sheet;
  gint retval,acol,arow;
  gboolean restore_active=FALSE;

  temp_sheet=sg_current_worksheet;
  sg_current_worksheet=worksheet;
  sheet = GTK_SHEET(worksheet->sheet);

  if (object)
  {
#ifdef WITH_NUMERIC_PYTHON
      if (PyArray_Check(object))
       { gtk_sheet_freeze(sheet);
         retval=python_array(worksheet,row,col,(PyArrayObject *)object,orient,link);
         gtk_sheet_thaw(sheet);
       }
      else
#endif
      if (PySequence_Check(object))
       { gtk_sheet_freeze(sheet);
         retval=python_sequence(worksheet,row,col,object,orient,link);
         gtk_sheet_thaw(sheet);
       }

      else
         retval=python_singleton(worksheet,row,col,object,link);

   }
  else
      retval=0;

  return retval;
}

PyObject *python_eval_expr(gchar *command)
{ PyObject *object;

  object=PyRun_String (command, Py_eval_input, dict_o, dict_o);
  if (object)
  return object;
   { if (PyErr_Occurred()) PyErr_Clear();
     PyRun_String (command, Py_single_input, dict_o, dict_o);
   }

  if (PyErr_Occurred())
      { if (child_pid>0)
           { PyErr_Print();
             if (Py_FlushLine()) PyErr_Clear();
           }
        PyErr_Clear();
      }
  return NULL;
}

int python_sheet(SGworksheet *worksheet,gint row, gint col, gchar *command,
                GtkOrientation orient)
{ PyObject *object=NULL;
  gchar errmsg[]="Error!";
  gchar *msg;
  GtkSheet *sheet;
  SGworksheet *temp_sheet;
  gint retval;

  temp_sheet=sg_current_worksheet;
  sg_current_worksheet=worksheet;
  sheet = GTK_SHEET(worksheet->sheet);


  object=PyRun_String (command, Py_eval_input, dict_o, dict_o);
  if (!object)
   { if (PyErr_Occurred()) PyErr_Clear();
     PyRun_String (command, Py_single_input, dict_o, dict_o);
   }


  if (object)
   {  Py_INCREF(object);
      retval=python_insert_object(worksheet,row, col, object, orient,FALSE);
      Py_XDECREF(object);
   }
  else
   retval=0;

  sg_current_worksheet=temp_sheet;
  if (!retval)
   { if (PyErr_Occurred())
      { msg=errmsg;
          if (child_pid>0)
           { PyErr_Print();
             if (Py_FlushLine()) PyErr_Clear();
           }
        PyErr_Clear();
      }
     else msg=command;
     gtk_sheet_set_cell_text(GTK_SHEET(sheet),row,col,msg);
     return 0;
   }
  return 1;
}
