/*  zterm.c - Zed's Virtual Terminal
 *  Copyright (C) 1998  Michael Zucchi
 *
 *  A simple terminal program, based on ZTerm.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 Adapted for use in sg by C. Steenberg 23/6/2000

 */

/* needed for getopt under 'gcc -ansi -pedantic' on GNU/Linux */
#ifndef _GNU_SOURCE
#  define _GNU_SOURCE 1
#endif

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <pwd.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkprivate.h>
#include <gdk/gdkkeysyms.h>

#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>


#include <readline/readline.h>
#include <readline/history.h>

#ifdef WITH_GNOME
#include <gnome.h>
#include <zvt/zvtterm.h>
#else
#include "zvtterm.h"
#endif
#include "../../config.h"
#include "python_command.h"
#include "../pixmaps/sg.xpm"

pid_t child_pid=-1;
gint stdout_pipe[2],stderr_pipe[2],line_pipe[2],com_pipe[2], life_pipe[2],
    stdout_tag, stderr_tag,line_tag, com_tag, life_tag,
    term_x,term_y,term_xmin=40,term_ymin=15,
    sizey_busy=0,restarted=0,term_open=0;
GtkWidget *layout, *hscrollbar, *vscrollbar, *container,*table;
GtkWidget *term, *hbox, *scrollbar;
FILE *stdout_fp,*fp;
extern gint py_status; /* 0=normal, 1=old line waiting */
extern GdkPixmap *icon_pixmap;
extern GdkBitmap *icon_mask;

#define FONT "-misc-fixed-medium-r-normal--12-200-75-75-c-100-iso8859-1"

extern char      **environ;		
static char      **env;
static char      **env_copy;
static int         winid_pos;
static GtkWidget  *window = NULL;

static void child_died();

void term_kill_child()
{   if (child_pid<0) return;
    if (waitpid(child_pid,NULL,WNOHANG)!=0)
    { kill(child_pid,9);
      waitpid(child_pid,NULL,WNOHANG);
    }
    term_open=0;
    py_status=0;
    gtk_widget_destroy(GTK_WIDGET(window));
    child_pid=-1;
}

void
child_died_event (ZvtTerm *term)
{   term_open=0;
    py_status=0;
    gtk_widget_destroy(GTK_WIDGET(window));
    window=NULL;
    child_pid=-1;
}

static void
title_changed_event (ZvtTerm *term, VTTITLE_TYPE type, char *newtitle)
{
  switch(type) {
  case VTTITLE_WINDOW:
  case VTTITLE_WINDOWICON:
    gtk_window_set_title (GTK_WINDOW (window), newtitle);
    break;
  default:
    break;
  }
}

static int
button_press_event (ZvtTerm *term, GdkEventButton *e)
{
  int x,y;
  GdkModifierType mask;
  char *data, *str;

  gdk_window_get_pointer(GTK_WIDGET(term)->window, &x, &y, &mask);

  str = zvt_term_match_check(term, x/term->charwidth, y/term->charheight,(void *)&data);
  if (str) {
    gtk_signal_emit_stop_by_name (GTK_OBJECT (term), "button_press_event");
    return TRUE;
  }
  return FALSE;
}

static void
set_hints (GtkWidget *widget)
{
        ZvtTerm *term;
	GdkGeometry hints;
	GtkWidget *app;

	g_assert (widget != NULL);
	term = ZVT_TERM (widget);

	app = gtk_widget_get_toplevel(widget);
	g_assert (app != NULL);

#define PADDING 2
	hints.base_width = (GTK_WIDGET (term)->style->klass->xthickness * 2) + PADDING;
	hints.base_height =  (GTK_WIDGET (term)->style->klass->ythickness * 2);

	hints.width_inc = term->charwidth;
	hints.height_inc = term->charheight;
	hints.min_width = hints.base_width + hints.width_inc;
	hints.min_height = hints.base_height + hints.height_inc;

	gtk_window_set_geometry_hints(GTK_WINDOW(app),
				      GTK_WIDGET(term),
				      &hints,
				      GDK_HINT_RESIZE_INC|GDK_HINT_MIN_SIZE|GDK_HINT_BASE_SIZE);
}

static void sg_term_line_typed (gpointer data, gint fd, GdkInputCondition condition)
{
  gboolean update;
  gchar buffer[4096],norm[]="R>>> ",cont[]="R... ";
  gint count, saveerrno,res,wres;
  ZvtTerm *term;

  term = (ZvtTerm *) data;

  if (term->input_id == -1)
    return;

  saveerrno = 0;
  
  while ( (count = read (fd, buffer, 4096)) > 0)
    { saveerrno = errno;
      switch (errno){   
      case   EBADF:fprintf(stderr,"EBADF\n"); break;
      case   EINVAL:fprintf(stderr,"EINVAL\n");break;
      case   EINTR:fprintf(stderr,"EINTR\n");  break;
      case   EFAULT:fprintf(stderr,"EFAULT\n");break;
      case   EPIPE:fprintf(stderr,"EPIPE\n");  break;
      case   EAGAIN:fprintf(stderr,"EAGAIN\n");break;
      case   EIO:fprintf(stderr,"EIO\n");
      }
      if (count<4096) break;
    }
// Just a test

  res=python_simple(buffer,count);

  if (res==2) wres=write(com_pipe[1],cont,strlen(cont));
  else wres=write(com_pipe[1],norm,strlen(norm));
  if (wres<0){
      fprintf(stderr,"Error writing prompt\n");
      switch (errno){
      case   EBADF:fprintf(stderr,"EBADF\n");
      case   EINVAL:fprintf(stderr,"EINVAL\n");
      case   EINTR:fprintf(stderr,"EINTR\n");  
      case   EFAULT:fprintf(stderr,"EFAULT\n");
      case   EPIPE:fprintf(stderr,"EPIPE\n");  
      case   EAGAIN:fprintf(stderr,"EAGAIN\n");
      case   ENOSPC:fprintf(stderr,"ENOSPC\n");
      case   EIO:fprintf(stderr,"EIO\n");
      }
  }
  return;
  
} 

void
print_raw (GtkWidget * textwidget, unsigned char *text, gint len)
{
   int dotime = FALSE;
   char num[8];
   int reverse = 0, under = 0, bold = 0,
       comma, k, i = 0, j = 0;
   unsigned char *newtext = malloc (len + 1024);

   while (i < len)
   { switch (text[i])
      {
      case '\026':		/// REVERSE
         if (reverse)
         {
            reverse = FALSE;
            strcpy (&newtext[j], "\033[27m");
         } else
         {
            reverse = TRUE;
            strcpy (&newtext[j], "\033[7m");
         }
         j = strlen (newtext);
         break;
      case '\037':		// underline
         if (under)
         {
            under = FALSE;
            strcpy (&newtext[j], "\033[24m");
         } else
         {
            under = TRUE;
            strcpy (&newtext[j], "\033[4m");
         }
         j = strlen (newtext);
         break;
      case '\002':		// bold
         if (bold)
         {
	      bold = FALSE;
	      strcpy (&newtext[j], "\033[22m");
         } else
 	   {
            bold = TRUE;
            strcpy (&newtext[j], "\033[1m");
         }
         j = strlen (newtext);
         break;
      case '\017':		// reset all
         strcpy (&newtext[j], "\033[0m");  
         j = strlen (newtext);
         reverse = FALSE;
         bold = FALSE;   
         under = FALSE;  
         break;
      case '\n':
          newtext[j++] = '\r';
      default:
         newtext[j] = text[i];
         j++;
      }
      i++;
   }
   newtext[j] = 0;
   zvt_term_feed ((ZvtTerm *) textwidget, newtext, j);
   free (newtext);
}

static void sg_term_write (gpointer data, gint fd, GdkInputCondition condition)
{
  gchar buffer[4096];
  gint count, saveerrno,*fd2;

  fd2=(gint *)data;

  saveerrno = EAGAIN;

  while ( (count = read (fd, buffer, 4096)) > 0) {
      saveerrno = errno;
      write (fd2[1], buffer, count);
      if (count<4096) break;
    }
}


static void sg_term_readdata (gpointer data, gint fd, GdkInputCondition condition)
{
  gchar buffer[4096];
  gint count, saveerrno;
  ZvtTerm *term;

  term = (ZvtTerm *) data;

  if (term->input_id == -1)
    return;

  saveerrno = EAGAIN;

  while ( (count = read (fd, buffer, 4096)) > 0) {
      saveerrno = errno;
      print_raw (GTK_WIDGET(term), buffer, count);
      if (count<4096) break;
    }
}

static void sg_term_life (gpointer data, gint fd, GdkInputCondition condition)
{
  gchar buffer[10];
  gint count, saveerrno;
  ZvtTerm *term;

  term = (ZvtTerm *) data;

  if (term->input_id == -1)
    return;

  saveerrno = EAGAIN;

  while ( (count = read (fd, buffer, 10)) > 0) {
      saveerrno = errno;
      if (count<10) break;
    }
}


static void parent_died()
{  _exit(0);
}


/* create_child: forks off a new process and handles input via readline
 on the newly created terminal. Todo: create python command completion
 code.
*/
int create_child(void)
{  gchar buffer [256],buf2[4096],*line;
   gint len,i,mychar;
   ZvtTerm *g_term;

  pipe(line_pipe); /* For the child to tell the parent what line was typed */
  pipe(life_pipe); /* For the child to see if parent is alive */
  pipe(com_pipe); /* For the parent to tell child what is what */
  g_term=ZVT_TERM(term);
  child_pid=vt_forkpty(&g_term->vx->vt,FALSE);
  switch (child_pid)
  {  case -1:
              perror ("Error: unable to fork");
              return 0;
              
      case 0:{signal(SIGPIPE,&parent_died);
              signal(SIGHUP,&parent_died);
              signal(SIGINT,SIG_IGN);
              signal(SIGQUIT,SIG_IGN);
              close(line_pipe[0]); /* Close read end */
              close(life_pipe[0]); /* Close read end */
              close(com_pipe[1]); /* Close write end */
              rl_reset_terminal("vt100");
              rl_initialize();
              rl_filename_completion_desired=0;
#ifdef READLINE_4
              rl_catch_signals=0;
              rl_free_line_state();
#endif
              for (;;)
             {  len=0;
                i=0;
                 do {
                     len=read(com_pipe[0],buffer,256);
                     if (i>10) {
                         write(line_pipe[1], "\n", 1);
                         usleep(100000);
                         i=0;
                     }
                 }
                while (len<=0);

                if (buffer[0]=='R') /* Only action defined so far */
                 {  buffer[len]= '\0';
                   line=0;
                   line=readline(buffer+1);
                    if (line) {
                     if (*line)
                         add_history(line);
                     len= strlen(line);
                     if (len) write(line_pipe[1], line, len);
                     else write(line_pipe[1], "\n", 1);
                     free(line);
                    }
                    else write(line_pipe[1], "\n", 1);
                 }
                if (buffer[0]=='K') break;
             }
          _exit(0);
          }
      default:
       { /*signal(SIGINT,&term_kill_child);
         signal(SIGQUIT,&term_kill_child);
         signal(SIGSEGV,&term_kill_child);
         signal(SIGPIPE,&term_kill_child);*/
         signal(SIGFPE, SIG_IGN);
         signal(SIGCHLD, &child_died);
/*         signal (SIGTSTP, SIG_DFL);
         signal (SIGTTIN, SIG_IGN);
         signal (SIGTTOU, SIG_IGN);*/
         pipe(stdout_pipe);
        stdout_tag=gdk_input_add(stdout_pipe[0], GDK_INPUT_READ,
                                 sg_term_readdata, term);
        sprintf(buf2,"sys.stdout=os.fdopen(%d,\"w\",1)",stdout_pipe[1]);
        PyRun_SimpleString (buf2);
        stdout_fp=fdopen(stdout_pipe[1],"w");
        pipe(stderr_pipe);
        stderr_tag=gdk_input_add(stderr_pipe[0], GDK_INPUT_READ,
                      sg_term_readdata, term);
        sprintf(buf2,"sys.stderr=os.fdopen(%d,\"w\",1)",stderr_pipe[1]);
        PyRun_SimpleString (buf2);
        close(line_pipe[1]); /* Close write end */
        close(life_pipe[1]);
        line_tag=gdk_input_add(line_pipe[0], GDK_INPUT_READ,
                         sg_term_line_typed, term);
        life_tag=gdk_input_add(life_pipe[0], GDK_INPUT_READ,
                         sg_term_life, term);
        close(com_pipe[0]); /* Close read end */
        fcntl(com_pipe[1],F_SETFL,O_NONBLOCK); /* non-blocking write-end*/
        write(com_pipe[1],"R>>> ",5);
        ZVT_TERM(term)->input_id =
        gdk_input_add(g_term->vx->vt.childfd, GDK_INPUT_READ,
                      sg_term_write, (gpointer)stdout_pipe);

       }
  }
}

static void child_died()
{ 
  if (stdout_tag) gdk_input_remove(stdout_tag);
  if (stderr_tag) gdk_input_remove(stderr_tag);
  if (line_tag) gdk_input_remove(line_tag);
  if (life_tag) gdk_input_remove(life_tag);
  if (ZVT_TERM(term)->input_id) gdk_input_remove(ZVT_TERM(term)->input_id);
  close(com_pipe[1]);
  close(stdout_pipe[0]);
  close(stderr_pipe[0]);
  close(line_pipe[0]);
  close(life_pipe[0]);
  waitpid(child_pid,NULL,WNOHANG);
  if (window) gtk_widget_destroy(GTK_WIDGET(window));
  child_pid=-1;
}


/*
  Do setup, initialises windows, forks child.
*/
void create_python_term(GtkWidget *widget, gpointer data)
{
  int i, c, cmdindex, scrollbacklines, login_shell;
  char **p, *fontname=FONT;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  enum { RIGHT, LEFT } scrollpos = RIGHT;

  if (term_open) return;
  else term_open=1;

  login_shell = 0;
  cmdindex = 0;
  scrollbacklines = 50;

  /* Create widgets and set options */
#ifdef WITH_GNOME
  window=gnome_app_new (PACKAGE,"Python terminal (SciGraphica)");
#else
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Python terminal (SciGraphica)");
  gtk_window_set_wmclass (GTK_WINDOW (window), "zterm", "zterm");
#endif
  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
                                                 gdk_colormap_get_system(),
                                                 &mask, NULL, sg_xpm);
  gtk_widget_realize (window);
  gdk_window_set_icon(window->window, NULL, pixmap, mask);
  /* create hbox */
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_set_spacing (GTK_BOX (hbox), 2);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
#ifdef WITH_GNOME
  gnome_app_set_contents(GNOME_APP(window),hbox);
#else
  gtk_container_add (GTK_CONTAINER (window), hbox);
#endif
  gtk_widget_show (hbox);

  /* create terminal */
  term = zvt_term_new_with_size(80,24);
  zvt_term_set_font_name(ZVT_TERM (term), fontname);
  zvt_term_set_blink (ZVT_TERM (term), TRUE);
  zvt_term_set_bell (ZVT_TERM (term), TRUE);
  zvt_term_set_scrollback(ZVT_TERM (term), scrollbacklines);
  zvt_term_set_scroll_on_keystroke (ZVT_TERM (term), TRUE);
  zvt_term_set_scroll_on_output (ZVT_TERM (term), FALSE);
  zvt_term_set_background (ZVT_TERM (term), NULL, 0, 0);
  zvt_term_set_wordclass (ZVT_TERM (term), "-A-Za-z0-9/_:.,?+%=");

  zvt_term_match_add (ZVT_TERM (term), "http://[^ ]*[^\\.]", VTATTR_UNDERLINE|(1<<VTATTR_FORECOLOURB)|(2<<VTATTR_BACKCOLOURB), "http");
  zvt_term_match_add (ZVT_TERM (term), "ftp://[^ ]*[^\\.]", VTATTR_BOLD, "ftp");
  zvt_term_match_add (ZVT_TERM (term), "file://[^ ]*[^\\.]", VTATTR_REVERSE, "file");
  zvt_term_match_add (ZVT_TERM (term), "[0-9\\.]*", VTATTR_BOLD, "ip #");

  gtk_signal_connect (
      GTK_OBJECT (term),
      "button_press_event",
      GTK_SIGNAL_FUNC (button_press_event),
      NULL);
  
  gtk_signal_connect (
      GTK_OBJECT (term),
      "child_died",
      GTK_SIGNAL_FUNC (child_died_event),
      NULL);

  gtk_signal_connect (
      GTK_OBJECT (window),
      "destroy",
      GTK_SIGNAL_FUNC (child_died_event),
      NULL);

  gtk_signal_connect (
      GTK_OBJECT (term),
      "title_changed",
      GTK_SIGNAL_FUNC (title_changed_event),
      NULL);
  
  gtk_signal_connect_after (
      GTK_OBJECT (term),
      "realize",
      GTK_SIGNAL_FUNC (set_hints),
      term);

  gtk_widget_show (term);

  /* scrollbar */
  scrollbar = 
    gtk_vscrollbar_new (GTK_ADJUSTMENT (ZVT_TERM (term)->adjustment));
  GTK_WIDGET_UNSET_FLAGS (scrollbar, GTK_CAN_FOCUS);
  if (scrollpos == LEFT) {
    gtk_box_pack_start (GTK_BOX (hbox), scrollbar, FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), term, 1, 1, 0);
  } else {
    gtk_box_pack_start (GTK_BOX (hbox), term, 1, 1, 0);
    gtk_box_pack_start (GTK_BOX (hbox), scrollbar, FALSE, TRUE, 0);
  }
  gtk_widget_show (scrollbar);
  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
                                                gdk_colormap_get_system(),
                                                &mask, NULL, sg_xpm);

  
  /* show them all! */
  gtk_widget_realize(window);
  gdk_window_set_icon(window->window, NULL, icon_pixmap, icon_mask);             

  gtk_widget_show (window);

  create_child();
}


