/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"

typedef struct{
  GtkWidget *parent_box;

  SGlayer *layer;
  SGplot *plot;

  GtkPlotAxis *axis;
  GtkPlotTicks *ticks;
  GtkWidget *from_entry, *to_entry, *major_entry, *first_entry;
  GtkWidget *minor_spin;
  GtkWidget *scale_combo, *rescale_combo;
  GtkWidget *axis_1, *axis_2;
  GtkWidget *labels_1, *labels_2;
  GtkWidget *titles_1, *titles_2;
  GtkWidget *major_1_in, *major_2_in;
  GtkWidget *major_1_out, *major_2_out;
  GtkWidget *minor_1_in, *minor_2_in;
  GtkWidget *minor_1_out, *minor_2_out;
  GtkWidget *minor_grids, *major_grids;
  GtkWidget *opposite, *origin;
  GtkOrientation orientation;
} SGaxisdialog;

static SGaxisdialog axis_dialog[2];

static gchar *scale[] = {"Linear", "Log10", NULL};
/*, "Ln", "Log2", NULL};
*/

static gchar *rescale[] = {"None", "Automatic", NULL};


void
sg_axis_dialog_update_plot(GtkWidget *notebook)
{
  SGaxisdialog *dialog;
  GtkPlot *plot;
  GtkWidget *child;
  gboolean state;
  gchar *text;
  gdouble max, min;
  gdouble major;
  gdouble begin, end;
  gboolean set_limits;
  gint nminor;
  gint page;
 
  page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
  dialog = &axis_dialog[page];
  plot = GTK_PLOT(dialog->layer->real_plot);
 
  min = atof(gtk_entry_get_text(GTK_ENTRY(dialog->from_entry))); 
  max = atof(gtk_entry_get_text(GTK_ENTRY(dialog->to_entry))); 

  if(min >= max) return;

  major = atof(gtk_entry_get_text(GTK_ENTRY(dialog->major_entry))); 

  if(major < 0) return;

  text = gtk_entry_get_text(GTK_ENTRY(dialog->first_entry));

  if(strlen(text) == 0){
    set_limits = FALSE;
    begin = min;
    end = max;
  }
  else{
    set_limits = TRUE;
    begin = atof(text);
    end = max;
  }

  switch(dialog->orientation){
    case GTK_ORIENTATION_VERTICAL:
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->axis_1));
       gtk_plot_axis_set_visible(plot, GTK_PLOT_AXIS_LEFT, state);

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->axis_2));
       gtk_plot_axis_set_visible(plot, GTK_PLOT_AXIS_RIGHT, state);

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->labels_1));
       plot->left.label_mask = state * GTK_PLOT_LABEL_LEFT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->labels_2));
       plot->right.label_mask = state * GTK_PLOT_LABEL_RIGHT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->titles_1));
       plot->left.title_visible = state;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->titles_2));
       plot->right.title_visible = state;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_1_out));
       plot->left.major_mask = (plot->left.major_mask & GTK_PLOT_TICKS_RIGHT) +
                                state * GTK_PLOT_TICKS_LEFT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_2_out));
       plot->right.major_mask = (plot->right.major_mask & GTK_PLOT_TICKS_LEFT) +
                                 state * GTK_PLOT_TICKS_RIGHT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_1_in));
       plot->left.major_mask = (plot->left.major_mask & GTK_PLOT_TICKS_LEFT) +
                                state * GTK_PLOT_TICKS_RIGHT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_2_in));
       plot->right.major_mask = (plot->right.major_mask & GTK_PLOT_TICKS_RIGHT)
                                 + state * GTK_PLOT_TICKS_LEFT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_1_out));
       plot->left.minor_mask = (plot->left.minor_mask & GTK_PLOT_TICKS_RIGHT) +
                                state * GTK_PLOT_TICKS_LEFT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_2_out));
       plot->right.minor_mask = (plot->right.minor_mask & GTK_PLOT_TICKS_LEFT) +
                                 state * GTK_PLOT_TICKS_RIGHT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_1_in));
       plot->left.minor_mask = (plot->left.minor_mask & GTK_PLOT_TICKS_LEFT) +
                                state * GTK_PLOT_TICKS_RIGHT;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_2_in));
       plot->right.minor_mask = (plot->right.minor_mask & GTK_PLOT_TICKS_RIGHT)
                                 + state * GTK_PLOT_TICKS_LEFT;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_grids));
       plot->show_hmajor = state;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_grids));
       plot->show_hminor = state;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->opposite));
       plot->show_x0 = state;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->origin));
       plot->show_y0 = state;

       child = GTK_LIST(GTK_COMBO(dialog->scale_combo)->list)->selection->data;
       plot->yscale = gtk_list_child_position(GTK_LIST(GTK_COMBO(dialog->scale_combo)->list), child);
       plot->right.scale_type = plot->yscale;
       plot->left.scale_type = plot->yscale;

       nminor = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dialog->minor_spin));
       gtk_plot_set_yrange(plot, min, max);
       gtk_plot_axis_set_ticks(plot, GTK_ORIENTATION_VERTICAL, major, nminor);
       plot->yticks.set_limits = set_limits;
       plot->yticks.begin = begin;
       plot->yticks.end = end;

       break;
    case GTK_ORIENTATION_HORIZONTAL:
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->axis_1));
       gtk_plot_axis_set_visible(plot, GTK_PLOT_AXIS_BOTTOM, state);

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->axis_2));
       gtk_plot_axis_set_visible(plot, GTK_PLOT_AXIS_TOP, state);

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->labels_1));
       plot->bottom.label_mask = state * GTK_PLOT_LABEL_BOTTOM;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->labels_2));
       plot->top.label_mask = state * GTK_PLOT_LABEL_TOP;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->titles_1));
       plot->bottom.title_visible = state;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->titles_2));
       plot->top.title_visible = state;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_1_out));
       plot->bottom.major_mask = (plot->bottom.major_mask & GTK_PLOT_TICKS_UP) +
                                  state * GTK_PLOT_TICKS_DOWN;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_2_out));
       plot->top.major_mask = (plot->top.major_mask & GTK_PLOT_TICKS_DOWN) +
                               state * GTK_PLOT_TICKS_UP;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_1_in));
       plot->bottom.major_mask = (plot->bottom.major_mask & GTK_PLOT_TICKS_DOWN)
                                  + state * GTK_PLOT_TICKS_UP;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_2_in));
       plot->top.major_mask = (plot->top.major_mask & GTK_PLOT_TICKS_UP) +
                               state * GTK_PLOT_TICKS_DOWN;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_1_out));
       plot->bottom.minor_mask = (plot->bottom.minor_mask & GTK_PLOT_TICKS_UP) +
                                  state * GTK_PLOT_TICKS_DOWN;
 
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_2_out));
       plot->top.minor_mask = (plot->top.minor_mask & GTK_PLOT_TICKS_DOWN) +
                               state * GTK_PLOT_TICKS_UP;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_1_in));
       plot->bottom.minor_mask = (plot->bottom.minor_mask & GTK_PLOT_TICKS_DOWN)
                                  + state * GTK_PLOT_TICKS_UP;

       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_2_in));
       plot->top.minor_mask = (plot->top.minor_mask & GTK_PLOT_TICKS_UP) +
                               state * GTK_PLOT_TICKS_DOWN;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->major_grids));
       plot->show_vmajor = state;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->minor_grids));
       plot->show_vminor = state;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->opposite));
       plot->show_y0 = state;
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->origin));
       plot->show_x0 = state;

       child = GTK_LIST(GTK_COMBO(dialog->scale_combo)->list)->selection->data;
       plot->xscale = gtk_list_child_position(GTK_LIST(GTK_COMBO(dialog->scale_combo)->list), child);
       plot->top.scale_type = plot->xscale;
       plot->bottom.scale_type = plot->xscale;

       nminor = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dialog->minor_spin));
       gtk_plot_set_xrange(plot, min, max);
       gtk_plot_axis_set_ticks(plot, GTK_ORIENTATION_HORIZONTAL, major, nminor);
       plot->xticks.set_limits = set_limits;
       plot->xticks.begin = begin;
       plot->xticks.end = end;
  }

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(dialog->plot->real_canvas));
}


static void
sg_build_axis_dialog (SGlayer *layer, GtkPlotAxis *axis, SGaxisdialog *dialog)
{
  GtkPlot *plot;
  GtkWidget *main_box;
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *box;
  GtkWidget *table;
  GtkWidget *align;
  GtkRequisition req;
  GtkAdjustment *adj;
  gchar text[20];
  gboolean state;
  static  gboolean h, v;


  dialog->layer = layer;
  dialog->plot = (SGplot *)layer->parent;
  plot = GTK_PLOT(layer->real_plot);
  dialog->orientation = axis->orientation;
  if(dialog->orientation == GTK_ORIENTATION_HORIZONTAL){
    h = TRUE;
    v = FALSE;
    dialog->ticks = &plot->xticks;
  }
  else
  {
    v = TRUE;
    h = FALSE;
    dialog->ticks = &plot->yticks;
  }

  /* Create widgets */
  main_box = dialog->parent_box = gtk_hbox_new (FALSE,5);
  gtk_container_set_border_width(GTK_CONTAINER(main_box), 10);

  frame = gtk_frame_new("Scale");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  box = gtk_vbox_new(FALSE, 5);
  gtk_container_add (GTK_CONTAINER (frame), box);
 
  table = gtk_table_new(8,2,FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table), 10);

  label = gtk_label_new("From:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1); 

  dialog->from_entry = gtk_entry_new();
  gtk_widget_size_request(dialog->from_entry, &req);
  req.width /= 2;
  gtk_widget_set_usize(dialog->from_entry, req.width, req.height);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->from_entry, 1, 2, 0, 1); 
  sprintf(text,"%f",axis->min);
  gtk_entry_set_text(GTK_ENTRY(dialog->from_entry), text); 
  sg_entry_set_numeric(GTK_ENTRY(dialog->from_entry), 15);

  label = gtk_label_new("To:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2); 
  dialog->to_entry = gtk_entry_new();
  gtk_widget_set_usize(dialog->to_entry, req.width, req.height);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->to_entry, 1, 2, 1, 2); 

  sprintf(text,"%f",axis->max);
  gtk_entry_set_text(GTK_ENTRY(dialog->to_entry), text); 
  sg_entry_set_numeric(GTK_ENTRY(dialog->to_entry), 15);

  label = gtk_label_new("Major ticks:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3); 
  dialog->major_entry = gtk_entry_new();
  gtk_widget_set_usize(dialog->major_entry, req.width, req.height);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->major_entry, 1, 2, 2, 3); 

  sprintf(text,"%f",dialog->ticks->step);
  gtk_entry_set_text(GTK_ENTRY(dialog->major_entry), text); 
  sg_entry_set_numeric(GTK_ENTRY(dialog->major_entry), 15);

  label = gtk_label_new("# minor ticks:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4); 
  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 100., 1., 1., 0.);

  dialog->minor_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_widget_set_usize(dialog->minor_spin, req.width, req.height);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->minor_spin, 1, 2, 3, 4); 

  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(dialog->minor_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(dialog->minor_spin), 0);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(dialog->minor_spin), dialog->ticks->nminor);


  label = gtk_label_new("1st tick:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 4, 5); 
  dialog->first_entry = gtk_entry_new();
  gtk_widget_set_usize(dialog->first_entry, req.width, req.height);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->first_entry, 1, 2, 4, 5); 
  sprintf(text,"%f",dialog->ticks->begin);
  if(dialog->ticks->set_limits)
         gtk_entry_set_text(GTK_ENTRY(dialog->first_entry), text); 

  sg_entry_set_numeric(GTK_ENTRY(dialog->first_entry), 15);

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_hseparator_new(), 0, 2, 5, 6); 

  label = gtk_label_new("Type:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 6, 7); 
  dialog->scale_combo = gtk_combo_new();
  gtk_widget_set_usize(dialog->scale_combo, req.width, req.height);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->scale_combo, 1, 2, 6, 7); 
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(dialog->scale_combo)->entry), FALSE);

  sg_combo_set_items(GTK_COMBO(dialog->scale_combo), scale);

  gtk_list_select_item(GTK_LIST(GTK_COMBO(dialog->scale_combo)->list),
                       axis->scale_type);

  label = gtk_label_new("Rescale:");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 7, 8); 
  dialog->rescale_combo = gtk_combo_new();
  gtk_widget_set_usize(dialog->rescale_combo, req.width, req.height);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->rescale_combo, 1, 2, 7, 8); 
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(dialog->rescale_combo)->entry), FALSE);

  sg_combo_set_items(GTK_COMBO(dialog->rescale_combo), rescale);

  gtk_list_select_item(GTK_LIST(GTK_COMBO(dialog->rescale_combo)->list),
                       layer->rescale);

  frame = gtk_frame_new("Display");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  box = gtk_vbox_new(FALSE, 2);
  gtk_container_add (GTK_CONTAINER (frame), box);

  table = gtk_table_new(8,6,FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
  gtk_table_set_col_spacings(GTK_TABLE(table), 2);
  gtk_table_set_row_spacings(GTK_TABLE(table), 2);

  if(h)
     sprintf(text,"%s","bottom");
  else
     sprintf(text,"%s","left");
  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(text),1,3,0,1);

  if(h)
     sprintf(text,"%s","top");
  else
     sprintf(text,"%s","right");
  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(text),3,5,0,1);

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_hseparator_new(), 0, 5, 1, 2); 

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("Axis"),0,1,2,3);

  align = gtk_alignment_new(.5, 0., 0., 0.);
  dialog->axis_1 = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->axis_1);
  gtk_table_attach_defaults(GTK_TABLE(table),align,1,3,2,3);
  state = (v && gtk_plot_axis_get_visible(plot, GTK_PLOT_AXIS_LEFT)) || 
          (h && gtk_plot_axis_get_visible(plot, GTK_PLOT_AXIS_BOTTOM)); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->axis_1), state);
  align = gtk_alignment_new(.5, 0., 0., 0.);
  dialog->axis_2 = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->axis_2);
  gtk_table_attach_defaults(GTK_TABLE(table),align,3,5,2,3);
  state = (v && gtk_plot_axis_get_visible(plot, GTK_PLOT_AXIS_RIGHT)) || 
          (h && gtk_plot_axis_get_visible(plot, GTK_PLOT_AXIS_TOP)); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->axis_2), state);


  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("Ticks labels"),0,1,3,4);

  align = gtk_alignment_new(.5, 0., 0., 0.);
  dialog->labels_1 = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->labels_1);
  gtk_table_attach_defaults(GTK_TABLE(table),align,1,3,3,4);
  state = (v && plot->left.label_mask & GTK_PLOT_LABEL_LEFT) || 
          (h && plot->bottom.label_mask & GTK_PLOT_LABEL_BOTTOM); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->labels_1), state);
  align = gtk_alignment_new(.5, 0., 0., 0.);
  dialog->labels_2 = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->labels_2);
  gtk_table_attach_defaults(GTK_TABLE(table),align,3,5,3,4);
  state = (v && plot->right.label_mask & GTK_PLOT_LABEL_RIGHT) || 
          (h && plot->top.label_mask & GTK_PLOT_LABEL_TOP); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->labels_2), state);

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("Titles"),0,1,4,5);

  align = gtk_alignment_new(.5, 0., 0., 0.);
  dialog->titles_1 = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->titles_1);
  gtk_table_attach_defaults(GTK_TABLE(table),align,1,3,4,5);
  state = (v && plot->left.title_visible) || 
          (h && plot->bottom.title_visible); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->titles_1), state);
  align = gtk_alignment_new(.5, 0., 0., 0.);
  dialog->titles_2 = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->titles_2);
  gtk_table_attach_defaults(GTK_TABLE(table),align,3,5,4,5);
  state = (v && plot->right.title_visible) || 
          (h && plot->top.title_visible); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->titles_2), state);



  gtk_table_attach_defaults(GTK_TABLE(table),gtk_hseparator_new(), 0, 5, 5, 6); 

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("Ticks"),0,1,6,7);

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("out"),1,2,6,7);
  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("in"),2,3,6,7);
  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("out"),3,4,6,7);
  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("in"),4,5,6,7);

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("Major"),0,1,7,8);
  gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new("Minor"),0,1,8,9);


  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->major_1_out = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->major_1_out);
  gtk_table_attach_defaults(GTK_TABLE(table),align,1,2,7,8);
  state = (v && plot->left.major_mask & GTK_PLOT_TICKS_LEFT) | 
          (h && plot->bottom.major_mask & GTK_PLOT_TICKS_DOWN); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->major_1_out), state);
  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->major_1_in = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->major_1_in);
  gtk_table_attach_defaults(GTK_TABLE(table),align,2,3,7,8);
  state = (v && plot->left.major_mask & GTK_PLOT_TICKS_RIGHT) | 
          (h && plot->bottom.major_mask & GTK_PLOT_TICKS_UP); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->major_1_in), state);
  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->major_2_out = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->major_2_out);
  gtk_table_attach_defaults(GTK_TABLE(table),align,3,4,7,8);
  state = (v && plot->right.major_mask & GTK_PLOT_TICKS_RIGHT) | 
          (h && plot->top.major_mask & GTK_PLOT_TICKS_UP); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->major_2_out), state);
  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->major_2_in = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->major_2_in);
  gtk_table_attach_defaults(GTK_TABLE(table),align,4,5,7,8);
  state = (v && plot->right.major_mask & GTK_PLOT_TICKS_LEFT) | 
          (h && plot->top.major_mask & GTK_PLOT_TICKS_DOWN); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->major_2_in), state);

  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->minor_1_out = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->minor_1_out);
  gtk_table_attach_defaults(GTK_TABLE(table),align,1,2,8,9);
  state = (v && plot->left.minor_mask & GTK_PLOT_TICKS_LEFT) | 
          (h && plot->bottom.minor_mask & GTK_PLOT_TICKS_DOWN); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->minor_1_out), state);
  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->minor_1_in = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->minor_1_in);
  gtk_table_attach_defaults(GTK_TABLE(table),align,2,3,8,9);
  state = (v && plot->left.minor_mask & GTK_PLOT_TICKS_RIGHT) | 
          (h && plot->bottom.minor_mask & GTK_PLOT_TICKS_UP); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->minor_1_in), state);
  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->minor_2_out = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->minor_2_out);
  gtk_table_attach_defaults(GTK_TABLE(table),align,3,4,8,9);
  state = (v && plot->right.minor_mask & GTK_PLOT_TICKS_RIGHT) |
          (h && plot->top.minor_mask & GTK_PLOT_TICKS_UP); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->minor_2_out), state);
  align = gtk_alignment_new(1., 0., 0., 0.);
  dialog->minor_2_in = gtk_check_item_new();
  gtk_container_add(GTK_CONTAINER(align), dialog->minor_2_in);
  gtk_table_attach_defaults(GTK_TABLE(table),align,4,5,8,9);
  state = (v && plot->right.major_mask & GTK_PLOT_TICKS_LEFT) | 
          (h && plot->top.minor_mask & GTK_PLOT_TICKS_DOWN); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->minor_2_in), state);

  gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), FALSE, FALSE, 0);

  table = gtk_table_new(5,2,FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
  gtk_table_set_col_spacings(GTK_TABLE(table), 2);
  gtk_table_set_row_spacings(GTK_TABLE(table), 2);

  if(axis->orientation == GTK_ORIENTATION_HORIZONTAL)
     sprintf(text,"%s","Vertical Grids");
  else
     sprintf(text,"%s","Horizontal Grids");

  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(text),0,2,0,1);

  dialog->major_grids = gtk_check_item_new_with_label("major");
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->major_grids,0,1,1,2);
  state = (v & plot->show_hmajor) || (h & plot->show_vmajor); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->major_grids), state);

  dialog->minor_grids = gtk_check_item_new_with_label("minor");
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->minor_grids,1,2,1,2);
  state = (v & plot->show_hminor) || (h & plot->show_vminor); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->minor_grids), state);

  gtk_table_attach_defaults(GTK_TABLE(table),gtk_hseparator_new(), 0, 4, 2, 3); 

  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new("Aditional lines"),0,4,3,4);

  dialog->opposite = gtk_check_item_new_with_label("opposite");
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->opposite,0,1,4,5);
  state = (v & plot->show_x0) || (h & plot->show_y0); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->opposite), state);

  if(axis->orientation == GTK_ORIENTATION_HORIZONTAL)
     sprintf(text,"%s","X=0");
  else
     sprintf(text,"%s","Y=0");


  dialog->origin = gtk_check_item_new_with_label(text);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->origin,1,2,4,5);
  state = (v & plot->show_y0) || (h & plot->show_x0); 
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->origin), state);

  /* Show widgets */
  gtk_widget_show_all (main_box);
}


GtkWidget *
sg_axis_dialog_new(SGlayer *layer)
{
  GtkWidget *notebook;
  GtkPlot *plot;

  notebook = gtk_notebook_new();

  plot = GTK_PLOT(layer->real_plot);
  active_layer = layer;
  sg_build_axis_dialog(layer, &plot->top, &axis_dialog[0]);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), axis_dialog[0].parent_box,
                           gtk_label_new("X axis"));
  sg_build_axis_dialog(layer, &plot->left, &axis_dialog[1]);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), axis_dialog[1].parent_box,
                           gtk_label_new("Y axis"));
  return notebook;
}
