/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "sg_worksheet.h"
#include "sg_clipboard.h"

static GList *selection;

void
sg_clipboard_cut(GtkSheet *sheet)
{
  gint row, col;
  gchar *text;
  GtkSheetCell *cell;
  SGhiddencell *link;

  sg_clipboard_clear();

  gtk_sheet_freeze(sheet);
  
  for(row = sheet->range.row0; row <= sheet->range.rowi; row++){
    for(col = sheet->range.col0; col <= sheet->range.coli; col++){

       text = gtk_sheet_cell_get_text(sheet, row, col);
       link = (SGhiddencell *)gtk_sheet_get_link(sheet, row, col);
       if(link) text = link->formula;

       if(text){
           cell = g_new(GtkSheetCell, 1);
           cell->text = g_strdup(text);
           cell->row = row - sheet->range.row0;
           cell->col = col - sheet->range.col0;
           gtk_sheet_cell_clear(sheet, row, col);
           selection = g_list_append(selection, cell);
       }
    }
  }

  gtk_sheet_thaw(sheet);
}

void
sg_clipboard_copy(GtkSheet *sheet)
{
  gint row, col;
  gchar *text;
  GtkSheetCell *cell;

  sg_clipboard_clear();
  
  gtk_sheet_freeze(sheet);

  for(row = sheet->range.row0; row <= sheet->range.rowi; row++){
    for(col = sheet->range.col0; col <= sheet->range.coli; col++){
       text = gtk_sheet_cell_get_text(sheet, row, col);
       if(text){
           cell = g_new(GtkSheetCell, 1);
           cell->text = g_strdup(text);
           cell->row = row - sheet->range.row0;
           cell->col = col - sheet->range.col0;
           selection = g_list_append(selection, cell);
       }
    }
  }

  gtk_sheet_thaw(sheet);
}

void
sg_clipboard_paste(GtkSheet *sheet)
{
  GtkSheetCell *cell;
  GList *list;

  gtk_sheet_freeze(sheet);

  list = selection;
  while(list){
    cell = (GtkSheetCell *)list->data;

    gtk_sheet_set_cell_text(sheet, 
                            sheet->range.row0 + cell->row,
                            sheet->range.col0 + cell->col,
                            cell->text);

    list = list->next;
  }
  gtk_sheet_thaw(sheet);
}

void
sg_clipboard_clear()
{
  GtkSheetCell *cell;
  GList *list;

  list = selection;
  while(list){
    cell = (GtkSheetCell *)list->data;
    g_free(cell->text);
    g_free(cell);

    selection = g_list_remove_link(selection, list);
    g_list_free_1(list);
    list = selection;
  }

  sg_clipboard_init();
}

void
sg_clipboard_init()
{
  selection = NULL;
}
