/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"

static GtkWidget *name_entry;
static GtkWidget *format_combo;
static GtkWidget *precision_spin;

static gint the_column;
static GtkSheet *the_sheet;

static gchar *formats[] = {"Number",
                           "Text",
                           "Date",
                           "Time",
		 	   NULL
};

static gboolean
mw_destroy(GtkWidget *widget)
{
  /* This is needed to get out of gtk_main */
  gtk_main_quit ();

  return FALSE;
}

static void
update_worksheet(GtkWidget *widget, gpointer data)
{
  GtkWidget *child;
  gchar *name;
  gint precision;
  gint format;

  name = gtk_entry_get_text(GTK_ENTRY(name_entry));

  gtk_sheet_set_column_title(GTK_SHEET(the_sheet), the_column, name);
  gtk_sheet_column_button_add_label(GTK_SHEET(the_sheet), the_column, name);

  child = GTK_LIST(GTK_COMBO(format_combo)->list)->selection->data;
  format = gtk_list_child_position(GTK_LIST(GTK_COMBO(format_combo)->list), child);

  precision = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(precision_spin));

  sg_worksheet_column_set_format(active_worksheet, the_column, 
                                 format,
                                 precision);


  gtk_widget_destroy(widget);
}


static void
format_changed(GtkWidget *widget, gpointer data)
{
  GtkWidget *child;
  gint format;
 
  child = GTK_LIST(GTK_COMBO(format_combo)->list)->selection->data;
  format = gtk_list_child_position(GTK_LIST(GTK_COMBO(format_combo)->list), child);

  if(format == SG_COLUMN_NUMBER)
      gtk_widget_set_sensitive(precision_spin, TRUE);
  else
      gtk_widget_set_sensitive(precision_spin, FALSE);

}


void
sg_column_dialog (SGworksheet *worksheet, gint col)
{
  GtkWidget *window = NULL;
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *main_box;
  GtkWidget *ok_button, *cancel_button;
  GtkWidget *action_area;
  GtkWidget *table;
  GtkSheetColumn *column;
  GtkAdjustment *adj;
  gchar text[100];

  the_column = col;
  the_sheet = GTK_SHEET(worksheet->sheet);
  column = &(GTK_SHEET(worksheet->sheet)->column[col]);

  window=gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
  gtk_window_set_title (GTK_WINDOW(window),"Column properties");

  gtk_window_set_modal (GTK_WINDOW(window),TRUE);

  /* Create widgets */
  main_box = gtk_vbox_new (FALSE,5);
  gtk_container_set_border_width(GTK_CONTAINER(main_box), 10);
  gtk_container_add (GTK_CONTAINER (window), main_box);

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);

  table = gtk_table_new(3, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 10);
  gtk_container_add(GTK_CONTAINER(frame), table);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  name_entry = gtk_entry_new();   

  label = gtk_label_new("Column name"),
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), 
                            label,
                            0, 1, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), name_entry, 1, 2, 0, 1);

  format_combo = gtk_combo_new();   

  label = gtk_label_new("Data format"),
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), 
                            gtk_label_new("Data format"),
                            0, 1, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), format_combo, 1, 2, 1, 2);

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(format_combo)->entry), FALSE);
  gtk_list_select_item(GTK_LIST(GTK_COMBO(format_combo)->list), 
                       worksheet->column[the_column].format); 


  label = gtk_label_new("Precision"),
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), 
                            gtk_label_new("Precision"),
                            0, 1, 2, 3);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 16., 1., 1., 0.);
  precision_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(precision_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(precision_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table), precision_spin, 1, 2, 2, 3);


  /* Action Area */
  action_area = gtk_hbutton_box_new ();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(action_area), 5);
  gtk_box_pack_end (GTK_BOX (main_box), action_area, FALSE, FALSE, 0);
  gtk_widget_show (action_area);

  ok_button = gtk_button_new_with_label ("OK");
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (action_area), ok_button, TRUE, TRUE, 0);
  gtk_widget_grab_default (ok_button);
  gtk_widget_show (ok_button);

  cancel_button = gtk_button_new_with_label ("Cancel");
  GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (action_area), cancel_button, TRUE, TRUE, 0);
  gtk_widget_show (cancel_button);

  /* connect signals */
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_destroy),
                             GTK_OBJECT (window));

  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
                             GTK_SIGNAL_FUNC (update_worksheet),
                             GTK_OBJECT (window));

  gtk_signal_connect_object (GTK_OBJECT (GTK_COMBO(format_combo)->entry), 
                             "changed",
                             GTK_SIGNAL_FUNC (format_changed),
                             GTK_OBJECT (window));

  gtk_signal_connect (GTK_OBJECT (window), "destroy",
                      GTK_SIGNAL_FUNC (mw_destroy),NULL);

  /* Show widgets */

  if(column->name)
    sprintf(text,"%s",column->name);
  else
    sprintf(text,"%d",col);

  gtk_entry_set_text(GTK_ENTRY(name_entry), text);

  sg_combo_set_items(GTK_COMBO(format_combo), formats);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(precision_spin), 
                            active_worksheet->column[the_column].precision);

  gtk_widget_show_all (window);

  /* wait until dialog get destroyed */
  gtk_main();

  return;
}
