/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"

static GtkWidget *style_combo1; 
static GtkWidget *style_combo2; 
static GtkWidget *thick_spin1; 
static GtkWidget *thick_spin2; 
static GtkWidget *color_combo1; 
static GtkWidget *color_combo2; 
static GtkPlot *plot;

static gchar *line_styles[] = {"None",
                              "Solid",
                              "Dotted",
                              "Dashed",
                              "Dot-dash",
                              "Dot-dot-dash",
                              "Dot-dash-dash",
                              NULL
                             };


void
sg_grids_dialog_update_plot(GtkWidget *widget)
{
  GtkColorCombo *combo;
  GdkColor color;
  gchar *color_name;
  SGplot *parent;
  GtkWidget *child;

  combo = GTK_COLOR_COMBO(color_combo1);
  color_name = gtk_color_combo_get_color_at(combo, combo->row, combo->column);
  gdk_color_parse(color_name, &color);
  gdk_color_alloc(color_combo1->style->colormap, &color);

  plot->major_vgrid.color = color;
  plot->major_vgrid.line_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(thick_spin1));

  child = GTK_LIST(GTK_COMBO(style_combo1)->list)->selection->data;
  plot->major_vgrid.line_style = gtk_list_child_position(GTK_LIST(GTK_COMBO(style_combo1)->list), child);

  combo = GTK_COLOR_COMBO(color_combo2);
  color_name = gtk_color_combo_get_color_at(combo, combo->row, combo->column);
  gdk_color_parse(color_name, &color);
  gdk_color_alloc(color_combo2->style->colormap, &color);

  plot->minor_vgrid.color = color;
  plot->minor_vgrid.line_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(thick_spin2));

  child = GTK_LIST(GTK_COMBO(style_combo2)->list)->selection->data;
  plot->minor_vgrid.line_style = gtk_list_child_position(GTK_LIST(GTK_COMBO(style_combo2)->list), child);

  plot->major_hgrid = plot->major_vgrid;
  plot->minor_hgrid = plot->minor_vgrid;

  parent = (SGplot *)active_layer->parent;
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(parent->real_canvas));
}


static void
init_dialog(GtkEntry *entry)
{
  GdkColor line_color;
  gint line_style, line_width;

  line_style = plot->major_vgrid.line_style;
  line_width = plot->major_vgrid.line_width;
  line_color = plot->major_vgrid.color;

  gtk_list_select_item(GTK_LIST(GTK_COMBO(style_combo1)->list), line_style);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(style_combo1)->entry), FALSE);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(thick_spin1), line_width);

  sg_color_combo_init(GTK_COLOR_COMBO(color_combo1), line_color);

  line_style = plot->minor_vgrid.line_style;
  line_width = plot->minor_vgrid.line_width;
  line_color = plot->minor_vgrid.color;

  gtk_list_select_item(GTK_LIST(GTK_COMBO(style_combo2)->list), line_style);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(style_combo2)->entry), FALSE);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(thick_spin2), line_width);

  sg_color_combo_init(GTK_COLOR_COMBO(color_combo2), line_color);
}


GtkWidget *
sg_grids_dialog_new (SGlayer *layer)
{
  GtkWidget *frame;
  GtkWidget *main_box, *main_table;
  GtkWidget *table;
  GtkRequisition req;
  GtkWidget *label;
  GtkAdjustment *adj;

  active_layer = layer;
  plot = GTK_PLOT(layer->real_plot);

  /* Create widgets */
  main_box = gtk_hbox_new (FALSE,5);
  gtk_container_set_border_width(GTK_CONTAINER(main_box), 5);

  main_table = gtk_table_new(2,2,FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(main_table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(main_table), 5);
  gtk_table_set_row_spacings(GTK_TABLE(main_table), 5);
  gtk_box_pack_start (GTK_BOX (main_box), main_table, FALSE, FALSE, 0);

  /* Major Grids */

  frame = gtk_frame_new("Major grids");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(main_table), frame, 0, 1, 0, 1);

  table=gtk_table_new(3, 3, FALSE);  
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
  gtk_container_add(GTK_CONTAINER(frame), table);

  label = gtk_label_new("Style ");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
  label = gtk_label_new("Thickness(pts) ");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
  label = gtk_label_new("Color ");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);


  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 6., 1., 1., 0.);
  thick_spin1 = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(thick_spin1), TRUE);
  style_combo1 = gtk_combo_new();
  sg_combo_set_items(GTK_COMBO(style_combo1), line_styles);
  color_combo1 = gtk_color_combo_new();

  gtk_widget_size_request(style_combo1, &req);
  req.width /= 2;
  gtk_widget_set_usize(style_combo1, req.width, req.height);
  gtk_widget_set_usize(thick_spin1, req.width, req.height);

  gtk_table_attach_defaults(GTK_TABLE(table), style_combo1, 1, 2, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), thick_spin1, 1, 2, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), color_combo1, 1, 2, 2, 3);

  /* Minor Grids */

  frame = gtk_frame_new("Minor grids");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_table_attach_defaults(GTK_TABLE(main_table), frame, 1, 2, 0, 1);

  table=gtk_table_new(3, 3, FALSE);  
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
  gtk_container_add(GTK_CONTAINER(frame), table);

  label = gtk_label_new("Style ");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
  label = gtk_label_new("Thickness(pts) ");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
  label = gtk_label_new("Color ");
  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);

  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 6., 1., 1., 0.);
  thick_spin2 = gtk_spin_button_new(adj, 1, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(thick_spin2), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(thick_spin2), 0);
  style_combo2 = gtk_combo_new();
  sg_combo_set_items(GTK_COMBO(style_combo2), line_styles);
  color_combo2 = gtk_color_combo_new();

  gtk_widget_set_usize(style_combo2, req.width, req.height);
  gtk_widget_set_usize(thick_spin2, req.width, req.height);

  gtk_table_attach_defaults(GTK_TABLE(table), style_combo2, 1, 2, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), thick_spin2, 1, 2, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), color_combo2, 1, 2, 2, 3);

  /* connect signals */
  gtk_signal_connect (GTK_OBJECT (color_combo1), "changed",
                             GTK_SIGNAL_FUNC (sg_color_combo_changed),
                             NULL);

  gtk_signal_connect (GTK_OBJECT (color_combo2), "changed",
                             GTK_SIGNAL_FUNC (sg_color_combo_changed),
                             NULL);

  init_dialog(NULL);
  return main_box;
}
