/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"

SGlayer *
sg_layer_new (gdouble width, gdouble height)
{
  SGlayer *layer;

  layer = g_new(SGlayer, 1);

  layer->real_plot = gtk_plot_new_with_size(NULL, width, height);

  layer->datasets = NULL;

  layer->button = gtk_toggle_button_new_with_label(" ");
  gtk_widget_set_usize(layer->button, 24, 24);

  GTK_PLOT_SET_FLAGS(layer->real_plot, GTK_PLOT_TRANSPARENT);
  return layer;
}

void
sg_layer_add_dataset(SGlayer *layer, SGdataset *dataset)
{
  gtk_plot_add_data(GTK_PLOT(layer->real_plot), dataset->real_data);

  layer->datasets = g_list_append(layer->datasets, dataset);
}


void
sg_layer_remove_dataset(SGlayer *layer, SGdataset *dataset)
{
  GList *list;

  list = layer->datasets;

  while(list) {
     if(list->data == dataset){
         gtk_plot_remove_data(GTK_PLOT(layer->real_plot),
                              dataset->real_data); 

         sg_dataset_destroy(dataset);
         layer->datasets = g_list_remove_link(layer->datasets, list);
         g_list_free_1(list);
         break;
     }
     list = list->next;
  }

}

void
sg_layer_button_set_label(SGlayer *layer, gint num)
{
  gchar title[10];
  
  sprintf(title,"%i\n",num);

  gtk_misc_set_alignment(GTK_MISC(GTK_BIN(layer->button)->child), .5, .0);
  gtk_label_set_text(GTK_LABEL(GTK_BIN(layer->button)->child), title);
}


