/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include "main.h"
#include "sg_toolbox_icons.h"

#define ZOOM 2.0000000000000
#define NTOP 2

static void     toolbox_autoscale                       (GtkButton *button,
                                                         gpointer data);
static void     toolbox_fitpage                         (GtkButton *button,
                                                         gpointer data);
static void     toolbox_fitpage_h                         (GtkButton *button,
                                                         gpointer data);
static void     toolbox_fitpage_v                         (GtkButton *button,
                                                         gpointer data);
static void     toolbox_up                              (GtkButton *button,
                                                         gpointer data);
static void     toolbox_down                            (GtkButton *button,
                                                         gpointer data);
static void     toolbox_left                            (GtkButton *button,
                                                         gpointer data);
static void     toolbox_right                           (GtkButton *button,
                                                         gpointer data);
static void     toolbox_zoomin                          (GtkButton *button,
                                                         gpointer data);
static void     toolbox_zoomout                         (GtkButton *button,
                                                         gpointer data);
static void     toolbox_select                          (GtkButton *button,
                                                         gpointer data);
static gint     tool_select_region                      (GtkPlotCanvas *canvas,
                                                         gdouble xmin,
                                                         gdouble xmax,
                                                         gdouble ymin,
                                                         gdouble ymax,
                                                         gpointer data);
static void     init_tool                               (SGplot *plot);




static void
toolbox_left(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;
  gdouble step;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  step = real_plot->xticks.step;

  gtk_plot_set_xrange(real_plot, 
                      real_plot->xmin - step, 
                      real_plot->xmax - step);

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));

}

static void
toolbox_right(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;
  gdouble step;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  step = real_plot->xticks.step;

  gtk_plot_set_xrange(real_plot, 
                      real_plot->xmin + step, 
                      real_plot->xmax + step);

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}

static void
toolbox_up(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;
  gdouble step;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  step = real_plot->yticks.step;

  gtk_plot_set_yrange(real_plot, 
                      real_plot->ymin + step, 
                      real_plot->ymax + step);

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}

static void
toolbox_down(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;
  gdouble step;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  step = real_plot->yticks.step;

  gtk_plot_set_yrange(real_plot, 
                      real_plot->ymin - step, 
                      real_plot->ymax - step);

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}

static void
toolbox_autoscale(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);
  gtk_plot_autoscale(real_plot);
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}

static void
toolbox_fitpage(GtkButton *button, gpointer data)
{
  sg_plot_fit_page((SGplot *)data);
}

static void
toolbox_fitpage_h(GtkButton *button, gpointer data)
{
  sg_plot_fit_page_h((SGplot *)data);
}

static void
toolbox_fitpage_v(GtkButton *button, gpointer data)
{
  sg_plot_fit_page_v((SGplot *)data);
}


static void
toolbox_zoomout(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  gtk_plot_set_yrange(real_plot, 
                      real_plot->ymin * ZOOM, 
                      real_plot->ymax * ZOOM);

  gtk_plot_set_xrange(real_plot, 
                      real_plot->xmin * ZOOM, 
                      real_plot->xmax * ZOOM);

  gtk_plot_axis_set_ticks(real_plot, GTK_ORIENTATION_HORIZONTAL,
                     real_plot->xticks.step * ZOOM,
                     real_plot->xticks.nminor);

  gtk_plot_axis_set_ticks(real_plot, GTK_ORIENTATION_VERTICAL,
                     real_plot->yticks.step * ZOOM,
                     real_plot->yticks.nminor);

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}

static void
toolbox_zoomin(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  gtk_plot_set_yrange(real_plot, 
                      real_plot->ymin / ZOOM, 
                      real_plot->ymax / ZOOM);

  gtk_plot_set_xrange(real_plot, 
                      real_plot->xmin / ZOOM, 
                      real_plot->xmax / ZOOM);

  gtk_plot_axis_set_ticks(real_plot, GTK_ORIENTATION_HORIZONTAL,
                     real_plot->xticks.step / ZOOM,
                     real_plot->xticks.nminor / ZOOM);

  gtk_plot_axis_set_ticks(real_plot, GTK_ORIENTATION_VERTICAL,
                     real_plot->yticks.step / ZOOM,
                     real_plot->yticks.nminor / ZOOM);

  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(plot->real_canvas));
}

static gint
tool_select_region(GtkPlotCanvas *canvas, 
                   gdouble x1, gdouble y1,
                   gdouble x2, gdouble y2,
                   gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;
  GtkPlotCanvasChild *child = NULL;
  gdouble xmin, ymin, xmax, ymax;
  gint px1, px2, py1, py2;

  xmin = MIN(x1, x2);
  ymin = MIN(y1, y2);
  xmax = MAX(x1, x2);
  ymax = MAX(y1, y2);

  plot = (SGplot *)data;

  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  switch(plot->tool){
    case SG_TOOL_ZOOM:
      gtk_plot_canvas_get_pixel(canvas, xmin, ymin, &px1, &py1);
      gtk_plot_canvas_get_pixel(canvas, xmax, ymax, &px2, &py2);
      gtk_plot_get_point(real_plot, px1, py1, &xmin, &ymax);
      gtk_plot_get_point(real_plot, px2, py2, &xmax, &ymin);
      gtk_plot_set_range(real_plot, xmin, xmax, ymin, ymax); 
      gtk_plot_canvas_refresh(canvas);
      return TRUE;
    case SG_TOOL_RECTANGLE:
      child = gtk_plot_canvas_put_rectangle(canvas, x1, y1, x2, y2,
                                            GTK_PLOT_LINE_SOLID, 0, 
                                            NULL, NULL, 1, FALSE);
      gtk_plot_canvas_refresh(canvas);
      return TRUE;
    case SG_TOOL_ELLIPSE:
      child = gtk_plot_canvas_put_ellipse(canvas, x1, y1, x2, y2,
                                          GTK_PLOT_LINE_SOLID, 0, 
                                          NULL, NULL, FALSE);
      gtk_plot_canvas_refresh(canvas);
      return TRUE;
    case SG_TOOL_LINE_ARROW:
      child = gtk_plot_canvas_put_line(canvas, x1, y1, x2, y2, 
                                       GTK_PLOT_LINE_SOLID, 0, NULL,
                                       GTK_PLOT_ARROW_END);
      gtk_plot_canvas_refresh(canvas);
      return TRUE;
    case SG_TOOL_LINE:
      child = gtk_plot_canvas_put_line(canvas, x1, y1, x2, y2, 
                                       GTK_PLOT_LINE_SOLID, 0, NULL,
                                       GTK_PLOT_ARROW_NONE);
      gtk_plot_canvas_refresh(canvas);
      return TRUE;
  }

  return FALSE;
}


static void
toolbox_select(GtkButton *button, gpointer data)
{
  SGplot *plot;
  GtkPlot *real_plot;
  SGtoolbox *toolbox;

  plot = (SGplot *)data;
  real_plot = GTK_PLOT(plot->active_layer->real_plot);

  toolbox = SG_TOOLBOX(plot->toolbox);

  GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                              GTK_PLOT_CANVAS_DND_FLAGS);

  if(button == GTK_BUTTON(toolbox->button[0][0])){
        plot->tool = SG_TOOL_ARROW;
        GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                  GTK_PLOT_CANVAS_DND_FLAGS);
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                    GTK_PLOT_CANVAS_CAN_DND_POINT |
                                    GTK_PLOT_CANVAS_CAN_SELECT |
                                    GTK_PLOT_CANVAS_CAN_SELECT_POINT);
  }
  if(button == GTK_BUTTON(toolbox->button[0][1])){
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                    GTK_PLOT_CANVAS_DND_FLAGS);
        GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                  GTK_PLOT_CANVAS_CAN_SELECT);
        plot->tool = SG_TOOL_ZOOM;
  }
  if(button == GTK_BUTTON(toolbox->button[1][0])){
        GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                  GTK_PLOT_CANVAS_DND_FLAGS);
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                    GTK_PLOT_CANVAS_CAN_SELECT |
                                    GTK_PLOT_CANVAS_CAN_SELECT_ITEM |
                                    GTK_PLOT_CANVAS_CAN_DND);
        plot->tool = SG_TOOL_POINTER;
  }

  if(button == GTK_BUTTON(toolbox->button[1][1])){
        plot->tool = SG_TOOL_TEXT;
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
				    GTK_PLOT_CANVAS_CAN_SELECT |
                                    GTK_PLOT_CANVAS_DND_FLAGS);
  }
  if(button == GTK_BUTTON(toolbox->button[2][0])){
        plot->tool = SG_TOOL_LINE;
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                    GTK_PLOT_CANVAS_DND_FLAGS);
        GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                  GTK_PLOT_CANVAS_CAN_SELECT);
  }
  if(button == GTK_BUTTON(toolbox->button[2][1])){
        plot->tool = SG_TOOL_LINE_ARROW;
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                    GTK_PLOT_CANVAS_DND_FLAGS);
        GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                  GTK_PLOT_CANVAS_CAN_SELECT);
  }
  if(button == GTK_BUTTON(toolbox->button[3][0])){
        plot->tool = SG_TOOL_RECTANGLE;
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                    GTK_PLOT_CANVAS_DND_FLAGS);
        GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                  GTK_PLOT_CANVAS_CAN_SELECT);
  }
  if(button == GTK_BUTTON(toolbox->button[3][1])){
        plot->tool = SG_TOOL_ELLIPSE;
        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                    GTK_PLOT_CANVAS_DND_FLAGS);
        GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(plot->real_canvas),
                                  GTK_PLOT_CANVAS_CAN_SELECT);
  }
}


void
sg_plot_toolbox_init(SGplot *plot)
{
 gint i, j;

 for(i = 0; i < 2; i++){
   for(j = 0; j < 4; j++){
      gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[j][i]),
                         "toggled",
                         GTK_SIGNAL_FUNC(toolbox_select), plot);
   }
 }

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[3+NTOP][0]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_zoomout), plot);

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[3+NTOP][1]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_zoomin), plot);

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[2+NTOP][1]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_fitpage), plot);

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[2+NTOP][0]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_autoscale), plot);

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[4+NTOP][0]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_up), plot);

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[4+NTOP][1]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_down), plot);

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[5+NTOP][0]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_left), plot);

 gtk_signal_connect(GTK_OBJECT(SG_TOOLBOX(plot->toolbox)->button[5+NTOP][1]),
                    "clicked",
                    GTK_SIGNAL_FUNC(toolbox_right), plot);

 gtk_signal_connect(GTK_OBJECT(plot->real_canvas), "select_region",
                    GTK_SIGNAL_FUNC(tool_select_region), plot);

 init_tool(plot);
}

static void
init_tool(SGplot *plot)
{
  gint row = 0, col = 0;

  switch(plot->tool){
   case SG_TOOL_ARROW:
      row = 0;
      col = 0;
      break;
   case SG_TOOL_ZOOM:
      row = 0;
      col = 1;
      break;
   case SG_TOOL_POINTER:
      row = 1;
      col = 0;
      break;
   case SG_TOOL_TEXT:
      row = 1;
      col = 1;
      break;
   case SG_TOOL_LINE:
      row = 2;
      col = 0;
      break;
   case SG_TOOL_LINE_ARROW:
      row = 2;
      col = 1;
      break;
   case SG_TOOL_RECTANGLE:
      row = 3;
      col = 0;
      break;
   case SG_TOOL_ELLIPSE:
      row = 3;
      col = 1;
      break;
  }
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SG_TOOLBOX(plot->toolbox)->button[row][col]), TRUE);
                           
}

