/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gtkextra/gtkextra.h>

#include "main.h"

void
sg_project_file_export_xml (gchar *filename)
{  
  GList *list;
  FILE *file;

  if ((file = fopen(filename, "w")) == NULL){
       g_warning("ERROR: Cannot open file: %s", filename);
       return;
  }

  fprintf(file,"<?xml version=\"1.0\"?>\n");
  fprintf(file,"<sg:Project xmlns:sg=\"http://scigraphica.sourceforge.net\">\n");

  fprintf(file, "  <sg:Summary>\n");
  fprintf(file, "    <sg:Item>\n");
  fprintf(file, "      <sg:name>application</sg:name>\n");
  fprintf(file, "      <sg:val-string>scigraphica</sg:val-string>\n");
  fprintf(file, "    </sg:Item>\n");
  fprintf(file, "    <sg:Item>\n");
  fprintf(file, "      <sg:name>author</sg:name>\n");
  fprintf(file, "      <sg:val-string>%s</sg:val-string>\n", g_get_real_name());
  fprintf(file, "    </sg:Item>\n");
  fprintf(file, "  </sg:Summary>\n");

  list = functions;
  while(list){
    SGdataset *aux;
 
    aux = (SGdataset *)list->data;

    fprintf(file,"  <sgf:Function Name=\"%s\" Exp=\"%s\"/>\n",
            aux->real_data->name, aux->exp);

    list = list->next;
  }

  list = worksheets;
  while(list){
    sg_worksheet_file_export_xml((SGworksheet *)list->data, filename, 
                                 NULL, file);

    list = list->next;
  }

  list = plots;
  while(list){
    sg_plot_file_export_xml((SGplot *)list->data, filename, file);

    list = list->next;
  }

  fprintf(file, "</sg:Project>\n");

  fclose(file);
}


